import re
from urllib.request import unquote, urlopen


def load_sync(path):
    if not re.match(r"^https?", path):
        path = "http://%s" % path
    path = unquote(path)
    return urlopen(path).read()



from remotecv.result_store import BaseStore
from remotecv.utils import logger


class ResultStore(BaseStore):

    WEEK = 604800
    memcache_instance = None

    def __init__(self, config):
        super(ResultStore, self).__init__(config)

        # pylibmc must be imported
        import pylibmc  # pylint: disable=import-error,import-outside-toplevel

        if not ResultStore.memcache_instance:
            host_strings = config.memcache_hosts.split(",")
            ResultStore.memcache_instance = pylibmc.Client(
                host_strings,
                binary=True,
                behaviors={"tcp_nodelay": True, "no_block": True, "ketama": True},
            )
        self.storage = ResultStore.memcache_instance

    def store(self, key, points):
        result = self.serialize(points)
        logger.debug("Points found: %s", result)
        key = "thumbor-detector-%s" % key
        self.storage.set(key, result, time=2 * self.WEEK)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# remote cv service
# https://github.com/globocom/thumbor/wiki

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 globo.com timehome@corp.globo.com

import argparse
import logging
import sys
from importlib import import_module

from redis import Redis

from remotecv.error_handler import ErrorHandler
from remotecv.utils import config


def start_pyres_worker():
    from remotecv.unique_queue import (  # NOQA pylint: disable=import-outside-toplevel
        UniqueWorker,
    )

    redis = Redis(
        host=config.redis_host,
        port=config.redis_port,
        db=config.redis_database,
        password=config.redis_password,
    )

    def after_fork(_):
        config.error_handler.install_handler()

    worker = UniqueWorker(
        queues=["Detect"], server=redis, timeout=config.timeout, after_fork=after_fork
    )
    worker.work()


def start_celery_worker():
    from remotecv.celery_tasks import (  # NOQA pylint: disable=import-outside-toplevel
        CeleryTasks,
    )

    celery_tasks = CeleryTasks(
        config.key_id,
        config.key_secret,
        config.region,
        config.timeout,
        config.polling_interval,
    )
    celery_tasks.run_commands(config.extra_args, log_level=config.log_level)


def main(params=None):
    if params is None:
        params = sys.argv[1:]
    parser = argparse.ArgumentParser(description="Runs RemoteCV.")

    conn_group = parser.add_argument_group("Worker Backend")
    conn_group.add_argument(
        "-b",
        "--backend",
        default="pyres",
        choices=["pyres", "celery"],
        help="Worker backend",
    )

    conn_group = parser.add_argument_group("Pyres Connection Arguments")
    conn_group.add_argument("--host", default="localhost", help="Redis host")
    conn_group.add_argument("--port", default=6379, type=int, help="Redis port")
    conn_group.add_argument("--database", default=0, type=int, help="Redis database")
    conn_group.add_argument("--password", default=None, help="Redis password")

    conn_group = parser.add_argument_group("Celery/SQS Connection Arguments")
    conn_group.add_argument("--region", default="us-east-1", help="AWS SQS Region")
    conn_group.add_argument("--key_id", default="", help="AWS access key id")
    conn_group.add_argument("--key_secret", default="", help="AWS access key secret")
    conn_group.add_argument(
        "--polling_interval", default=20, help="AWS polling interval"
    )

    other_group = parser.add_argument_group("Other arguments")
    other_group.add_argument("-l", "--level", default="debug", help="Logging level")
    other_group.add_argument(
        "-o", "--loader", default="remotecv.http_loader", help="Loader used"
    )
    other_group.add_argument(
        "-s", "--store", default="remotecv.result_store.redis_store", help="Loader used"
    )
    other_group.add_argument(
        "-t",
        "--timeout",
        default=None,
        type=int,
        help="Timeout in seconds for image detection",
    )
    other_group.add_argument(
        "--sentry_url", default=None, help="URL used to send errors to sentry"
    )

    memcache_store_group = parser.add_argument_group("Memcache store arguments")
    memcache_store_group.add_argument(
        "--memcache_hosts",
        default="localhost:11211",
        help="Comma separated list of memcache hosts",
    )

    parser.add_argument("args", nargs=argparse.REMAINDER)

    arguments = parser.parse_args(params)
    logging.basicConfig(level=getattr(logging, arguments.level.upper()))

    config.backend = arguments.backend
    config.redis_host = arguments.host
    config.redis_port = arguments.port
    config.redis_database = arguments.database
    config.redis_password = arguments.password

    config.region = arguments.region
    config.key_id = arguments.key_id
    config.key_secret = arguments.key_secret
    config.polling_interval = arguments.polling_interval

    config.timeout = arguments.timeout
    config.log_level = arguments.level.upper()
    config.loader = import_module(arguments.loader)
    config.store = import_module(arguments.store)

    config.memcache_hosts = arguments.memcache_hosts

    config.extra_args = sys.argv[:1] + arguments.args

    config.error_handler = ErrorHandler(arguments.sentry_url)

    if arguments.backend == "pyres":
        start_pyres_worker()
    elif arguments.backend == "celery":
        start_celery_worker()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

from remotecv.detectors.complete_detector import CompleteDetector
from remotecv.detectors.face_detector import FaceDetector
from remotecv.detectors.feature_detector import FeatureDetector
from remotecv.detectors.glasses_detector import GlassesDetector
from remotecv.detectors.profile_detector import ProfileDetector
from remotecv.image import Image


class ImageProcessor:
    def __init__(self):
        self.detectors = {
            "all": CompleteDetector(),
            "face": FaceDetector(),
            "feature": FeatureDetector(),
            "glass": GlassesDetector(),
            "profile": ProfileDetector(),
        }

    def detect(self, detector, image_data):
        result = []
        image = Image.create_from_buffer(image_data)
        if image is None:
            return []

        for detector_key in detector.split("+"):
            try:
                result = result + self.detectors[detector_key].detect(image)
            except KeyError:
                raise AttributeError("Detector unavailable")
        return result



import pkgutil
import sys

import pkg_resources
import sentry_sdk
from sentry_sdk.integrations.logging import ignore_logger
from sentry_sdk.utils import event_from_exception

from remotecv import __version__


class ErrorHandler:
    def __init__(self, sentry_url):
        self.sentry_client = None

        if not sentry_url:
            return

        kwargs = {
            "dsn": sentry_url,
            "integrations": [],
        }

        # Logging integration will create duplicates if below not ignored
        ignore_logger("thumbor")
        ignore_logger("tornado.access")

        sentry_sdk.init(**kwargs)
        self.extra_data = {"modules": self.get_modules()}
        self.install_handler()

    def get_modules(self):
        resolved = {}
        modules = [mod[1] for mod in tuple(pkgutil.iter_modules())]
        for module in modules:
            try:
                res_mod = pkg_resources.get_distribution(module)
                if res_mod is not None:
                    resolved[module] = res_mod.version
            except pkg_resources.DistributionNotFound:
                pass

        return resolved

    def install_handler(self):
        if not self.sentry_client:
            return
        old_hook = sys.excepthook

        def on_exception(exception_type, value, traceback):
            try:
                self.handle_error(exception_type, value, traceback)
            finally:
                old_hook(type, value, traceback)

        sys.excepthook = on_exception

    def handle_error(self, exception_type, value, traceback):
        exc_info = (exception_type, value, traceback)

        with sentry_sdk.push_scope() as scope:
            event, hint = event_from_exception(exc_info)
            scope.set_extra("remotecv-version", __version__)
            sentry_sdk.capture_event(event, hint=hint)



import json


class BaseStore:
    def __init__(self, config):
        self.config = config

    def serialize(self, points):
        result_array = []
        for point in points:
            result = {
                "x": point[0] + (int(point[2]) / 2),
                "y": point[1] + (int(point[3]) / 2),
                "height": int(point[2]),
                "width": int(point[3]),
                "origin": "",
            }
            result["z"] = result["width"] * result["height"]
            result_array.append(result)

        return json.dumps(result_array)

    def store(self, key, points):
        raise NotImplementedError()



from redis import Redis

from remotecv.result_store import BaseStore
from remotecv.utils import logger


class ResultStore(BaseStore):

    WEEK = 604800
    redis_instance = None

    def __init__(self, config):
        super(ResultStore, self).__init__(config)

        if not ResultStore.redis_instance:
            ResultStore.redis_instance = Redis(
                host=config.redis_host,
                port=config.redis_port,
                db=config.redis_database,
                password=config.redis_password,
            )
        self.storage = ResultStore.redis_instance

    def store(self, key, points):
        result = self.serialize(points)
        logger.debug("Points found: %s", result)
        redis_key = "thumbor-detector-%s" % key
        self.storage.setex(redis_key, result, 2 * self.WEEK)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['remotecv',
 'remotecv.detectors',
 'remotecv.detectors.complete_detector',
 'remotecv.detectors.face_detector',
 'remotecv.detectors.feature_detector',
 'remotecv.detectors.glasses_detector',
 'remotecv.detectors.profile_detector',
 'remotecv.result_store']

package_data = \
{'': ['*']}

install_requires = \
['Pillow>=7.0.0,<8.0.0',
 'opencv-python-headless>=4.2.0,<5.0.0',
 'pyres>=1.5,<2.0']

entry_points = \
{'console_scripts': ['remotecv = remotecv.worker:main']}

setup_kwargs = {
    'name': 'remotecv',
    'version': '3.0.0',
    'description': 'remotecv is an OpenCV worker for facial and feature recognition',
    'long_description': None,
    'author': 'Bernardo Heynemann',
    'author_email': 'heynemann@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# thumbor imaging service
# https://github.com/globocom/thumbor/wiki

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 globo.com timehome@corp.globo.com

from io import BytesIO
from os.path import abspath, dirname, join

import cv2
import numpy as np


class BaseDetector:
    def __get_format(self, image):
        fmt = image.format
        if fmt == "GIF":
            return "PNG"
        return fmt

    def get_np_img(self, image):
        img_buffer = BytesIO()
        image.save(img_buffer, self.__get_format(image))
        results = img_buffer.getvalue()
        img_buffer.close()
        nparr = np.fromstring(results, np.uint8)
        return cv2.imdecode(nparr, cv2.IMREAD_UNCHANGED)  # pylint: disable=no-member

    def detect(self, image):
        raise NotImplementedError()


class CascadeLoaderDetector(BaseDetector):
    def load_cascade_file(self, module_path, cascade_file_path):
        cascade_file = join(abspath(dirname(module_path)), cascade_file_path)
        self.__class__.cascade = cv2.CascadeClassifier(  # pylint: disable=no-member
            cascade_file
        )

    def get_min_size_for(self, size):
        ratio = int(min(size[0], size[1]) / 15)
        ratio = max(20, ratio)
        return (ratio, ratio)

    def get_features(self, image):
        img = self.get_np_img(image)

        faces = self.__class__.cascade.detectMultiScale(
            img, 1.2, 4, minSize=self.get_min_size_for(image.size)
        )
        faces_scaled = []

        for (left, top, width, height) in faces:
            faces_scaled.append(
                ((left.item(), top.item(), width.item(), height.item()), 0)
            )

        return faces_scaled

    def detect(self, image):
        features = self.get_features(image)

        if features:
            points = [
                [left, top, width, height]
                for (left, top, width, height), neighbors in features
            ]
        else:
            points = []

        return points



#!/usr/bin/python
# -*- coding: utf-8 -*-

# thumbor imaging service
# https://github.com/globocom/thumbor/wiki

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 globo.com timehome@corp.globo.com

import cv2
import numpy as np

from remotecv.detectors import BaseDetector

# pylint: disable=no-member


class FeatureDetector(BaseDetector):
    def detect(self, image):
        img = self.get_np_img(image)
        gray = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)
        corners = cv2.goodFeaturesToTrack(gray, 25, 0.01, 10)
        if corners is None:
            return None

        corners = np.int0(corners)

        if corners is not None and len(corners) > 0:
            points = []
            for corner in corners:  # pylint: disable=not-an-iterable
                left, top = corner.ravel()
                points.append([left, top, 1, 1])

            return points

        return None
